#include "NetworkMessage.h"
#include "Chunk.h"
#include "Entity.h"
#include "Game.h"

NetworkMessage::NetworkMessage()
{
	adress = 0;
	adressLength = 0;
	broadcast = 0;
	message = 0;
	msgDelete = 0;
	msgLength = 0;
	useBackground = 0;
	minPosition.x = std::numeric_limits<float>::min();
	minPosition.y = std::numeric_limits<float>::min();
	minPosition.z = std::numeric_limits<float>::min();
	maxPosition.x = std::numeric_limits<float>::max();
	maxPosition.y = std::numeric_limits<float>::max();
	maxPosition.z = std::numeric_limits<float>::max();
	affectedDimension = -1;
}

NetworkMessage::~NetworkMessage()
{
	if (msgDelete)
		delete[] message;
	delete[] adress;
}

void NetworkMessage::adressChunck(Chunk* zChunk)
{
	delete[] adress;
	adressLength = 10;
	adress = new char[adressLength];
	adress[0] = 1; // dimension response
	adress[1] = 1; // chunck
	Framework::Punkt center = zChunk->getCenter();
	*(int*)(adress + 2) = center.x;
	*(int*)(adress + 6) = center.y;
	minPosition = zChunk->getMin();
	maxPosition = zChunk->getMax();
	affectedDimension = zChunk->getDimensionId();
}

void NetworkMessage::adressEntity(Entity* zEntity)
{
	delete[] adress;
	adressLength = 6;
	adress = new char[adressLength];
	adress[0] = 1; // dimension response
	adress[1] = 2; // entity
	*(int*)(adress + 2) = zEntity->getId();
	minPosition = zEntity->getPosition();
	maxPosition = zEntity->getPosition();
	affectedDimension = zEntity->getCurrentDimensionId();
}

void NetworkMessage::adressBlock(Block* zBlock)
{
	delete[] adress;
	adressLength = 14;
	adress = new char[adressLength];
	adress[0] = 1; // dimension response
	adress[1] = 3; // block
	Framework::Vec3<int> pos = zBlock->getPos();
	*(int*)(adress + 2) = pos.x;
	*(int*)(adress + 6) = pos.y;
	*(int*)(adress + 10) = pos.z;
	minPosition = pos;
	maxPosition = pos;
	affectedDimension = zBlock->getDimensionId();
}

void NetworkMessage::openDialog(Framework::Text dialogName)
{
	delete[] adress;
	adressLength = (char)(4 + dialogName.getLength());
	adress = new char[adressLength];
	adress[0] = 2; // gui message
	adress[1] = 0; // open dialog
	*(short*)(adress + 2) = (short)dialogName.getLength(); // block
	memcpy(adress + 4, dialogName.getText(), dialogName.getLength());
}

void NetworkMessage::adressGui(Framework::Text elementId)
{
	delete[] adress;
	adressLength = (char)(4 + elementId.getLength());
	adress = new char[adressLength];
	adress[0] = 2; // gui message
	adress[1] = 1; // element message
	*(short*)(adress + 2) = (short)elementId.getLength(); // block
	memcpy(adress + 4, elementId.getText(), elementId.getLength());
}

void NetworkMessage::setMessage(char* msg, int length, bool deleteMsg)
{
	if (msgDelete)
		delete[] message;
	message = msg;
	msgLength = length;
	msgDelete = deleteMsg;
}

void NetworkMessage::setUseBackground()
{
	useBackground = 1;
}

void NetworkMessage::sendToAll()
{
	broadcast = true;
}

bool NetworkMessage::isAreaAffected(Framework::Vec3<float> min, Framework::Vec3<float> max, int affectedDimension) const
{
	return minPosition.x <= max.x && maxPosition.x >= min.x &&
		minPosition.y <= max.y && maxPosition.y >= min.y &&
		minPosition.z <= max.z && maxPosition.z >= min.z && (this->affectedDimension < 0 || this->affectedDimension == affectedDimension);
}

void NetworkMessage::writeTo(Framework::StreamWriter* zWriter) const
{
	int total = msgLength + adressLength;
	if (total)
	{
		zWriter->schreibe((char*)&GameClient::Message::API_MESSAGE, 1);
		zWriter->schreibe((char*)&total, 4);
		zWriter->schreibe(adress, adressLength);
		zWriter->schreibe(message, msgLength);
	}
}

bool NetworkMessage::isBroadcast() const
{
	return broadcast;
}

bool NetworkMessage::isEmpty() const
{
	return msgLength + adressLength <= 0;
}

bool NetworkMessage::isUseBackground() const
{
	return useBackground;
}